# 列表选择

> 以用户选择为例

1. 页面通过弹窗打开
2. 页面支持单选与多选
3. 页面关闭后返回选择结果

完整选择页面代码代码

```html
<!doctype html>
<html xmlns:th="http://www.thymeleaf.org">
<head>
    <base th:href="${#request.getContextPath() + '/'}">
    <meta charset="utf-8">
    <meta name="renderer" content="webkit">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=0">
    <link rel="stylesheet" href="assets/fonts/font-awesome/css/font-awesome.css"/>
    <link rel="stylesheet" href="assets/js/vendor/viewer/viewer.min.css">
    <link rel="stylesheet" href="assets/layuiadmin/layui/css/layui.css" media="all">
    <link rel="stylesheet" href="assets/layuiadmin/style/admin.css" media="all">
</head>
<body>
<div class="layui-anim layui-anim-fadein">
    <div class="layui-row">
        <div class="layui-col-md12">
            <div class="layui-card">
                <div class="layui-card-body">
                    <div class="test-table-reload-btn" style="margin-bottom: 10px;">
                        <div class="layui-inline">
                            <input type="text" id="keywords" class="layui-input search-item" placeholder="输入用户名、昵称">
                        </div>
                        最后登录
                        <div class="layui-inline">
                            <input type="text" id="startTime" class="layui-input search-item search-item-width-100" placeholder="开始时间" readonly>
                        </div>
                        <div class="layui-inline">
                            <input type="text" id="endTime" class="layui-input search-item search-item-width-100" placeholder="结束时间" readonly>
                        </div>
                        <div class="layui-inline layui-form search-item">
                            <select id="userType" name="userType" lay-filter="typeFilter">
                                <option value="">选择类型</option>
                                <option value="1">系统用户</option>
                                <option value="2">普通用户</option>
                                <option value="3">公共用户</option>
                            </select>
                        </div>
                        <div class="layui-inline layui-form search-item">
                            <select id="userState" name="userState" lay-filter="typeFilter">
                                <option value="">选择类型</option>
                                <option value="0">正常</option>
                                <option value="1">锁定</option>
                                <option value="-1">未审核</option>
                                <option value="-2">审核不通过</option>
                            </select>
                        </div>
                        <button type="button" id="search" class="layui-btn layui-btn-sm">
                            <i class="fa fa-lg fa-search"></i> 搜索
                        </button>
                        <button type="button" class="layui-btn layui-btn-sm layui-btn-normal" id="confirm" style="float:right;">确定</button>
                    </div>
                    <table class="layui-hide" id="dataTable" lay-filter="dataTable"></table>
                </div>
            </div>
        </div>
    </div>
</div>
<script src="assets/layuiadmin/layui/layui.js"></script>
<script th:inline="javascript">
    layui.config({
        base: 'assets/layuiadmin/'
    }).extend({
        index: 'lib/index'
    }).use(['index', 'table', 'laydate', 'upload'], function() {
        var $ = layui.$;
        var $win = $(window);
        var table = layui.table;
        var admin = layui.admin;
        var layer = layui.layer;
        var laydate = layui.laydate;
        var selectType = top.restAjax.params(window.location.href).selectType;
        selectType = selectType ? selectType : 'radio';
        // 已经选择的列表
        var selectedIdArray = top.dialog.dialogData.selectedIdArray ? top.dialog.dialogData.selectedIdArray : [];
        // 将新的选择列表置空
        var newSelectedArray = top.dialog.dialogData.newSelectedArray = [];
        // 正在选择的用户
        var selectUserMap = {};
        var tableData = [];

        var tableUrl = 'api/teacher/listpage-user-unselected';

        // 初始化已选择的列表
        function initSelectUserMap() {
            for(var i = 0, item; item = selectedIdArray[i++];) {
                selectUserMap[item] = {
                    userId: item
                }
            }
        }
        initSelectUserMap();

        // 初始化表格
        function initTable() {
            $.extend(table, {config: {checkName: 'checked'}});
            table.render({
                elem: '#dataTable',
                id: 'dataTable',
                url: top.restAjax.path(tableUrl, []),
                width: admin.screen() > 1 ? '100%' : '',
                height: $win.height() - 60,
                limit: 20,
                limits: [20, 40, 60, 80, 100, 200],
                toolbar: '#headerToolBar',
                request: {
                    pageName: 'page',
                    limitName: 'rows'
                },
                cols: [
                    [
                        {type: selectType, fixed: 'left'},
                        {field:'rowNum', width:80, title: '序号', fixed: 'left', align:'center', templet: '<span>{{d.LAY_INDEX}}</span>'},
                        {field:'userUsername', width:140, title: '用户名', sort: true, align:'center',
                            templet: function(rowData) {
                                return '<a href="javascript:void(0);" lay-event="userUsernameEvent">'+ rowData.userUsername +'</a>';
                            }
                        },
                        {field:'userName', width:140, title: '昵称', sort: true, align:'center'},
                        {field:'userType', width:90, title: '类型', sort: true, align:'center',
                            templet: function(item) {
                                if(item.userType == 1) {
                                    return '<span class="layui-badge layui-bg-green">系统用户</span>';
                                } else if(item.userType == 2) {
                                    return '<span class="layui-badge layui-bg-orange">普通用户</span>';
                                } else if(item.userType == 3) {
                                    return '<span class="layui-badge layui-bg-gray">公共用户</span>';
                                }
                                return '<span class="layui-badge">类型错误</span>';
                            }
                        },
                        {field:'userState', width:80, title: '状态', sort: true, align:'center',
                            templet: function(item) {
                                var value;
                                switch (item.userState) {
                                    case 1:
                                        value = '<span class="layui-badge layui-bg-blue">锁定</span>';
                                        break;
                                    case -1:
                                        value = '<span class="layui-badge layui-bg-gray">未审核</span>';
                                        break;
                                    case -2:
                                        value = '<span class="layui-badge">审核不通过</span>';
                                        break;
                                    default:
                                        value = '<span class="layui-badge layui-bg-green">正常</span>';
                                }
                                return value;
                            }
                        },
                        {field:'userPhone', width:140, title: '手机', sort: true, align:'center',
                            templet: function(item) {
                                if(!item.userPhone) {
                                    return '-';
                                }
                                return item.userPhone;
                            }
                        },
                        {field:'userEmail', width: 160, title: '邮箱', sort: true, align:'center',
                            templet: function(item) {
                                if(!item.userEmail) {
                                    return '-';
                                }
                                return item.userEmail;
                            }
                        },
                        {field:'userExpiredDate', width:180, title: '账号过期时间', align:'center',
                            templet: function(item) {
                                if(!item.userExpiredDate) {
                                    return '-';
                                }
                                return item.userExpiredDate;
                            }
                        },
                        {field:'gmtCreate', width:180, title: '创建时间', align:'center',
                            templet: function(item) {
                                if(!item.gmtCreate) {
                                    return '-';
                                }
                                return item.gmtCreate;
                            }
                        },
                    ]
                ],
                page: true,
                parseData: function(data) {
                    for(var i = 0, item; item = data.rows[i++];) {
                        if(selectUserMap[item.userId]) {
                            item.checked = true;
                            // 重新赋值：在打开界面后直接确定关闭页面时回显正常
                            selectUserMap[item.userId] = item;
                        } else {
                            item.checked = false;
                        }
                    }
                    tableData = data.rows;

                    return {
                        'code': 0,
                        'msg': '',
                        'count': data.total,
                        'data': data.rows
                    };
                }
            });
        }
        // 重载表格
        function reloadTable(currentPage) {
            table.reload('dataTable', {
                url: top.restAjax.path(tableUrl, []),
                where: {
                    keywords: $('#keywords').val(),
                    startTime: $('#startTime').val(),
                    endTime: $('#endTime').val(),
                    userType: $('#userType').val(),
                    userState: $('#userState').val()
                },
                page: {
                    curr: currentPage
                },
                height: $win.height() - 90,
            });
        }
        // 初始化日期
        function initDate() {
            // 日期选择
            laydate.render({
                elem: '#startTime',
                format: 'yyyy-MM-dd'
            });
            laydate.render({
                elem: '#endTime',
                format: 'yyyy-MM-dd'
            });
        }
        initTable();
        initDate();

        /**
         * 清空选择的用户
         */
        function clearSelect() {
            selectUserMap = {};
        }

        /**
         * 添加选择的用户
         * @param item
         */
        function addSelect(item) {
            selectUserMap[item.userId] = item;
        }

        /**
         * 删除选择的用户
         * @param item
         */
        function removeSelect(item) {
            delete selectUserMap[item.userId];
        }

        // 勾选事件
        table.on('checkbox(dataTable)', function(obj) {
            if(obj.type === 'all') {
                if(obj.checked) {
                    // 添加全部
                    for(var i = 0, item; item = tableData[i++];) {
                        addSelect(item);
                    }
                } else {
                    // 删除全部
                    for(var i = 0, item; item = tableData[i++];) {
                        removeSelect(item);
                    }
                }
            } else {
                if(obj.checked) {
                    addSelect(obj.data);
                } else {
                    removeSelect(obj.data);
                }
            }
        });

        // 勾选事件
        table.on('radio(dataTable)', function(obj) {
            clearSelect()
            addSelect(obj.data);
        });

        // 关闭
        function close() {
            parent.layer.close(parent.layer.getFrameIndex(window.name));
        }

        // 事件 - 搜索
        $(document).on('click', '#search', function() {
            reloadTable(1);
        });

        // 事件 - 搜索
        $(document).on('click', '#confirm', function() {
            for(var key in selectUserMap) {
                newSelectedArray.push(selectUserMap[key]);
            }
            close();
        });
    });
</script>
</body>
</html>
```

打开页面

```javascript
// 已经选择的ID列表
top.dialog.dialogData.selectedIdArray = [$('#userId').val()];
top.dialog.open({
    // 打开页面，指定选择框类型
    url: 'route/teacher/list-user-unselected?selectType=radio',
    title: '选择用户',
    width: '1000px',
    height: '500px',
    onClose: function() {
        // 获取选择后的列表
        var newSelectedArray = top.dialog.dialogData.newSelectedArray;
        if(newSelectedArray.length != 0) {
            // 处理选择结果
            $('#userId').val(newSelectedArray[0].userId);
            $('#userName').val(newSelectedArray[0].userName);
        }
    }
});
```